﻿// See https://aka.ms/new-console-template for more information


using Newtonsoft.Json;
using System.Net.Http.Headers;
using Serilog;
using CSharpFunctionalExtensions;

namespace FtpAgent
{
    public class OcrServer
    {
        public static string ServerLog { get; set; } = "upload.log";
        public static string OCR_SERVER_URL { get; private set; } = "https://hardware-test.guke.tech";
        public static string OCR_API_URL { get; private set; } = "/api/service/ocr/v2/recognize";
        [ErrorHandler<OcrResultData>("上传图片到OCR服务器失败!")]
        public static async Task<Result<OcrResultData, ApiServerError>> UploadFile(OcrRequest result)
        {
            File.AppendAllText(ServerLog, "图片: " + result.ocrFile +"\n");
            string url = $"{OCR_SERVER_URL}{OCR_API_URL}?ocrChannel=1&ocrType=4&remark=certificate test";
            using (var httpclient = new HttpClient())
            {
                var formData = new MultipartFormDataContent();
                formData.AddFile(result.ocrFile);
                formData.AddJson(result);
                Log.Information("Post请求:{url},上传文件:{file},请求参数:", url, result.ocrFile);
                await LogFormDataContent(formData);
                var response = await httpclient.PostAsync(url, formData);
                var responseContent = await response.Content.ReadAsStringAsync();
                if (!response.IsSuccessStatusCode)
                {
                    Log.Error($"请求远程OCR服务器失败，失败code=${response.StatusCode}, msg={responseContent}");
                    return Result.Failure<OcrResultData, ApiServerError>(new ApiServerError() { Code = (int)response.StatusCode, Msg = responseContent });
                }
                Log.Information("Post请求:{url},请求结果:{responseContent}", url, responseContent);
                var respond = JsonConvert.DeserializeObject<Respond<OcrResultData>>(responseContent);
                if (respond.code != 200)
                {
                    Log.Error($"请求远程OCR API失败，失败code=${respond.code}, msg={respond.detail}");
                    return Result.Failure<OcrResultData, ApiServerError>(new ApiServerError() { Code = respond.code, Msg = respond.msg });
                }
                var text = respond?.data?.lines.Select(item => item.text).Aggregate((l, f) => l + " " + f);
                Log.Information($"返回结果文本长度:{text.Length}");
                Log.Information($"拼接后的识别结果:{text}");
                File.AppendAllText(ServerLog, "内容: "+text +"\n");
                return Result.Success<OcrResultData, ApiServerError>(respond.data);
            }
        }

        public static async Task LogFormDataContent(MultipartFormDataContent formData)
        {
            var logData = new Dictionary<string, object>();

            foreach (var content in formData)
            {
                ContentDispositionHeaderValue disposition = content.Headers.ContentDisposition;
                string name = disposition.Name?.Trim('"');
                string? fileName = disposition.FileName?.Trim('"');

                if (content is StringContent || content.Headers.ContentType?.MediaType == "text/plain")
                {
                    string value = await content.ReadAsStringAsync();
                    logData.Add(name, value);
                }
                else if (fileName != null)
                {
                    logData.Add(name, $"(File: {fileName})");
                }
                else
                {
                    logData.Add(name, $"(Content-Type: {content.Headers.ContentType})");
                }
            }

            string json = JsonConvert.SerializeObject(logData, Formatting.Indented);
            Log.Information("Form Data Content: {FormData}", json);
        }
    }
}
