﻿// See https://aka.ms/new-console-template for more information


using CaptureAgent;
using FtpAgent;
using FubarDev.FtpServer;
using FubarDev.FtpServer.AccountManagement;
using FubarDev.FtpServer.FileSystem.DotNet;
using Microsoft.Extensions.DependencyInjection;
using Serilog;
using System.Reflection;

Logging.InitLogging();

Log.Information($"FTP Agent 启动");
var config = new Config("config.ini");
// 获取当前程序集
Assembly assembly = Assembly.GetExecutingAssembly();

// 获取版本信息
Version version = assembly.GetName().Version;
string informationalVersion = assembly.GetCustomAttribute<AssemblyInformationalVersionAttribute>().InformationalVersion;

Log.Information($"程序版本:{informationalVersion}");
Log.Information($"客户名: {config.CustomerName}");
Log.Information($"设备号: {config.DeviceNo}");
Log.Information($"OCR 类型: {config.OcrType}");
Log.Information($"订单号: {config.OrderNo}");
Log.Information($"订单类型: {config.OrderType}");
Log.Information($"拍照模式: {config.PhotoMode}");
Log.Information($"Secret: {config.Secret}");
Log.Information($"租户 Id: {config.TenantId}");
Log.Information($"OCR 服务器 URL: {config.OcrServerUrl}");
Log.Information($"OCR API URL: {config.OcrApiUrl}");
Log.Information($"IP: {config.Ip}");
Log.Information($"端口: {config.Port}");
Log.Information($"图片目录: {config.Dir}");
Log.Information($"是否上传: {config.Upload}");
Log.Information($"是否匿名: {config.Anonymous}");


// Setup dependency injection
var services = new ServiceCollection();

// use %TEMP%/TestFtpServer as root folder
services.Configure<DotNetFileSystemOptions>(opt => opt
    .RootPath = Path.Combine(Environment.CurrentDirectory, config.Dir));

// Add FTP server services
// DotNetFileSystemProvider = Use the .NET file system functionality
// AnonymousMembershipProvider = allow only anonymous logins
// Add FTP server services
services.AddFtpServer(builder =>
{
    builder.UseDotNetFileSystem(); // Use the .NET file system functionality
    if (config.Anonymous)
    {
        // Enable anonymous logins
        builder.EnableAnonymousAuthentication();
    }
    else
    {
        // Register your custom membership provider
        services.AddSingleton<IMembershipProvider, TestMembershipProvider>();
    }
});



// Configure the FTP server
services.Configure<FtpServerOptions>(opt =>
{
    opt.ServerAddress = config.Ip;
    opt.Port = config.Port;
});

// Build the service provider
using (var serviceProvider = services.BuildServiceProvider())
{
    Log.Information("开始运行FTP服务");

    Directory.CreateDirectory(config.Dir);

    // Initialize the FTP server
    var ftpServerHost = serviceProvider.GetRequiredService<IFtpServerHost>();

    // 设置文件系统监控器
    var watcher = new FileSystemWatcher(config.Dir);
    // 监控的文件类型
    watcher.Filter = "*.*";
    // 添加事件处理器
    watcher.Created += OnFileCreated;
    watcher.Changed += Watcher_Changed;
    Log.Information("添加文件监控器");
    // 开始监控
    watcher.EnableRaisingEvents = true;
    // Start the FTP server
    await ftpServerHost.StartAsync(CancellationToken.None);
    Log.Information("输入q退出");
    while (Console.ReadKey().Key != ConsoleKey.Q)
    {
        
    }
    Log.Information("停止FTP服务");
    // Stop the FTP server
    ftpServerHost?.StopAsync(CancellationToken.None).Wait();
    watcher.Dispose();
}




async void Watcher_Changed(object sender, FileSystemEventArgs e)
{
    Log.Information("{Name} 文件接收完成!",e.Name);
    if (!config.Upload)
    {
        return;
    }
    //如果被修改的是jpg文件，则转发到OCR服务器
    var ext = Path.GetExtension(e.Name);
    if (ext == ".jpg")
    {
        Log.Information("{Name} 文件转发到服务器", e.Name);
        OcrRequest result = new OcrRequest(e.FullPath)
        {
            deviceNo = config.DeviceNo,
            customerName = config.CustomerName,
            ocrType = config.OcrType,
            orderNo = config.OrderNo,
            orderType = config.OrderType,
            photoMode = config.PhotoMode,
            secret = config.Secret,
            tenantId = config.TenantId,
        };
        await OcrServer.UploadFile(result);
    }
    if (ext == ".txt")
    {
        Log.Information("{Name} 读码文件，数据为：", e.Name);
        Log.Information(File.ReadAllText(e.FullPath));
    }
}

void OnFileCreated(object sender, FileSystemEventArgs e)
{
    Log.Information($"文件:{e.Name} 被上传");
}
